/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.binding.soap.dfault.wsa;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.om.OMNode;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPHeader;
import org.bluestemsoftware.specification.eoa.component.intrface.rt.ActionContext.MessageModules;
import org.bluestemsoftware.specification.eoa.ext.binding.soap.SOAPVersion;
import org.bluestemsoftware.specification.eoa.ext.binding.soap.rt.SOAPFault;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.addressing.WSA;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.addressing.WSAMessageModule;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.addressing.WSA.WSA10;

public abstract class WSAHandler {

    /**
     * Gets instance which implements rules defined within verion implied by indicated
     * namespace.
     * @param namespace
     * @return
     */
    public static WSAHandler getInstance(String namespace) {
        if (namespace.equals(WSA.WSA10.NAMESPACE_URI)) {
            return WSA10Handler.getInstance();
        } else {
            throw new IllegalArgumentException("Namespace not supported");
        }
    }

    /**
     * Gets instance based upon addressing namespace used within headers defined on a request
     * received on behalf of my service, or a response recieved from a partner service.
     * <p>
     * Note that if message has no soap header or no addressing header blocks, the action is
     * undefined, which implies we have no binding module metadata. We use WSA10 as default
     * namespace to qualify fault name.
     * 
     * @param message
     * @return
     * @throws SOAPFault
     */
    public static WSAHandler getInstance(SOAPEnvelope message) throws SOAPFault {

        String envelopeNamespace = message.getVersion().getEnvelopeURI();
        SOAPVersion sv = SOAPVersion.valueOfEnvelopeNamespace(envelopeNamespace);
        SOAPHeader soapHeader = message.getHeader();

        if (soapHeader == null) {
            throw new HeaderRequiredFaultImpl(sv, WSA10.NAMESPACE_URI, WSA.ACTION);
        }

        String matchingNamespace = null;

        OMNode node = soapHeader.getFirstElement();

        while (node != null && node.getType() == OMNode.ELEMENT_NODE) {
            OMNamespace namespace = ((OMElement)node).getNamespace();
            if (namespace == null) {
                continue;
            }
            if (namespace.getNamespaceURI().equals(WSA10.NAMESPACE_URI)) {
                matchingNamespace = WSA10.NAMESPACE_URI;
                break;
            }
            node = node.getNextOMSibling();
        }

        if (matchingNamespace == null) {
            throw new HeaderRequiredFaultImpl(sv, WSA10.NAMESPACE_URI, WSA.ACTION);
        }

        return WSAHandler.getInstance(matchingNamespace);

    }

    /**
     * Parses wsa message headers contained with request envelope and returns result as an
     * abstract wsa message module.
     * @param message
     * @return
     * @throws SOAPFault
     */
    public abstract WSAMessageModule handleMyServiceRequest(SOAPEnvelope message) throws SOAPFault;

    /**
     * 
     * @param action
     * @param requestModules
     * @param message
     * @return address to which reply should be sent
     * @throws SOAPFault
     */
    public abstract WSAMessageModule handleMyServiceResponse(MessageModules requestModules, MessageModules responseModules, SOAPEnvelope message) throws SOAPFault;

    public abstract void handleWSAFaultResponse(String to, String relatesTo, String action, SOAPEnvelope message);
    
    public abstract void handlePartnerServiceRequest(MessageModules requestModules, SOAPEnvelope message) throws SOAPFault;

    /**
     * Parses wsa message headers contained with response envelope and returns result as an
     * abstract wsa message module.
     * @param message
     * @return
     * @throws SOAPFault
     */
    public abstract WSAMessageModule handlePartnerServiceResponse(SOAPEnvelope message) throws SOAPFault;

}
